import 'dart:async';
import 'package:flutter/services.dart';

typedef void SurveyWallOpenedListener();
typedef void SurveyWallDismissedListener();
typedef void DidReceiveRewardListener(int reward);
typedef void IsSurveyWallAvailableListener(int surveyAvailable);

class TapResearch {
  static TapResearch get instance => _instance;

  final MethodChannel _channel;

  static final TapResearch _instance = TapResearch.private(
    const MethodChannel('ios_native'),
  );

  TapResearch.private(MethodChannel channel) : _channel = channel {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  static SurveyWallOpenedListener _surveyWallOpenedListener;
  static SurveyWallDismissedListener _surveyWallDismissedListener;
  static DidReceiveRewardListener _didReceiveRewardListener;
  static IsSurveyWallAvailableListener _isSurveyWallAvailableListener;

  Future<void> init({String apiToken, String userId}) async {
    assert(apiToken != null && apiToken.isNotEmpty);

    return _channel.invokeMethod("init", <String, dynamic>{
      'api_token': apiToken,
      'user_id': userId,
    });
  }

  Future<void> initPlacement({String placementId}) async {
    return _channel.invokeMethod("initPlacement", <String, dynamic>{
      'placement_id': placementId,
    });
  }

  Future<void> showSurveyWall() async {
    try {
      await _channel.invokeMethod('showSurveyWall');
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> setNavigationBarText(String navBarText) async {
    try {
      await _channel.invokeMethod('setNavigationBarText', navBarText);
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> setNavigationBarColor(String hexColor) async {
    try {
      await _channel.invokeMethod('setNavigationBarColor', hexColor);
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> setNavigationBarTextColor(String hexColor) async {
    try {
      await _channel.invokeMethod('setNavigationBarTextColor', hexColor);
    } catch (e) {
      print(e.toString());
    }
  }

  Future _platformCallHandler(MethodCall call) async {
    print(
        "TapResearch _platformCallHandler call ${call.method} ${call.arguments}");

    switch (call.method) {
      case "tapResearchSurveyWallOpened":
        _surveyWallOpenedListener();
        break;

      case "tapResearchSurveyWallDismissed":
        _surveyWallDismissedListener();
        break;

      case "tapResearchDidReceiveReward":
        _didReceiveRewardListener(call.arguments);
        break;

      case "isSurveyWallAvailable":
        _isSurveyWallAvailableListener(call.arguments);
        break;

      default:
        print('Unknown method ${call.method} ');
    }
  }

  void setSurveyWallOpenedListener(
          SurveyWallOpenedListener surveyWallOpenedListener) =>
      _surveyWallOpenedListener = surveyWallOpenedListener;

  void setSurveyWallDismissedListener(
          SurveyWallDismissedListener surveyWallDismissedListener) =>
      _surveyWallDismissedListener = surveyWallDismissedListener;

  void setDidReceiveRewardListener(
          DidReceiveRewardListener didReceiveRewardListener) =>
      _didReceiveRewardListener = didReceiveRewardListener;

  void setIsSurveyWallAvailableListener(
          IsSurveyWallAvailableListener isSurveyWallAvailableListener) =>
      _isSurveyWallAvailableListener = isSurveyWallAvailableListener;
}
